Package: PhyloProfile
Version: 2.2.0
Date: 2025-10-20
Title: PhyloProfile
Authors@R: c(
    person("Vinh", "Tran", role = c("aut", "cre"), email = "tran@bio.uni-frankfurt.de", comment=c(ORCID="0000-0001-6772-7595")),
    person("Bastian", "Greshake Tzovaras", role = "aut", email = "bgreshake@googlemail.com"),
    person("Ingo", "Ebersberger", role = "aut", email = "ebersberger@bio.uni-frankfurt.de"),
    person("Carla", "Mölbert", role = "ctb", email = "carla.moelbert@gmx.de"))
Description: PhyloProfile is a tool for exploring complex phylogenetic profiles. Phylogenetic profiles, presence/absence patterns of genes over a set of species, are commonly used to trace the functional and evolutionary history of genes across species and time. With PhyloProfile we can enrich regular phylogenetic profiles with further data like sequence/structure similarity, to make phylogenetic profiling more meaningful. Besides the interactive visualisation powered by R-Shiny, the package offers a set of further analysis features to gain insights like the gene age estimation or core gene identification.
URL: https://github.com/BIONF/PhyloProfile/
BugReports: https://github.com/BIONF/PhyloProfile/issues
License: MIT + file LICENSE
Depends: R (>= 4.5.0)
Encoding: UTF-8
biocViews: Software, Visualization, DataRepresentation,
        MultipleComparison, FunctionalPrediction, DimensionReduction
Imports: ape, bioDist, BiocStyle, Biostrings, bsplus, colourpicker,
        data.table, dplyr, DT, energy, fastcluster, ggplot2, gridExtra,
        htmlwidgets, pbapply, plotly, RColorBrewer, RCurl, Rfast,
        scattermore, shiny, shinycssloaders, shinyFiles, shinyjs,
        stringr, tsne, svglite, umap, xml2, zoo, yaml
RoxygenNote: 7.3.3
Suggests: knitr, rmarkdown, testthat, OmaDB
VignetteBuilder: knitr
git_url: https://git.bioconductor.org/packages/PhyloProfile
git_branch: RELEASE_3_22
git_last_commit: aa891b5
git_last_commit_date: 2025-10-29
Repository: Bioconductor 3.22
Date/Publication: 2025-10-29
NeedsCompilation: no
Packaged: 2025-10-30 05:42:51 UTC; biocbuild
Author: Vinh Tran [aut, cre] (ORCID: <https://orcid.org/0000-0001-6772-7595>),
  Bastian Greshake Tzovaras [aut],
  Ingo Ebersberger [aut],
  Carla Mölbert [ctb]
Maintainer: Vinh Tran <tran@bio.uni-frankfurt.de>
