# Generated by roxygen2: do not edit by hand

export(addRankDivisionPlot)
export(calcPresSpec)
export(checkColorPalette)
export(checkInputValidity)
export(checkNewick)
export(checkOmaID)
export(clusterDataDend)
export(compareMedianTaxonGroups)
export(compareTaxonGroups)
export(createArchiPlot)
export(createDimRedPlotData)
export(createGeneAgePlot)
export(createLongMatrix)
export(createPercentageDistributionData)
export(createProfileFromOma)
export(createUnrootedTree)
export(createVarDistPlot)
export(createVariableDistributionData)
export(createVariableDistributionDataSubset)
export(dataCustomizedPlot)
export(dataFeatureTaxGroup)
export(dataMainPlot)
export(dataVarDistTaxGroup)
export(dimReduction)
export(estimateGeneAge)
export(fastaParser)
export(featureDistTaxPlot)
export(filterProfileData)
export(fromInputToProfile)
export(geneAgePlotDf)
export(generateSinglePlot)
export(getAllDomainsOma)
export(getAllFastaOma)
export(getCommonAncestor)
export(getCoreGene)
export(getDataClustering)
export(getDataForOneOma)
export(getDendrogram)
export(getDistanceMatrix)
export(getFastaFromFasInput)
export(getFastaFromFile)
export(getFastaFromFolder)
export(getIDsRank)
export(getInputTaxaID)
export(getInputTaxaName)
export(getNameList)
export(getOmaDataForOneOrtholog)
export(getOmaDomainFromURL)
export(getOmaMembers)
export(getQualColForVector)
export(getSelectedFastaOma)
export(getSelectedTaxonNames)
export(getTaxonomyInfo)
export(getTaxonomyMatrix)
export(getTaxonomyRanks)
export(gridArrangeSharedLegend)
export(groupLabelDimRedData)
export(heatmapPlotting)
export(heatmapPlottingFast)
export(highlightProfilePlot)
export(id2name)
export(linearizeArchitecture)
export(mainTaxonomyRank)
export(parseDomainInput)
export(parseInfoProfile)
export(plotDimRed)
export(plotDimRed3D)
export(prepareDimRedData)
export(processNcbiTaxonomy)
export(processOrthoID)
export(reduceProfile)
export(runPhyloProfile)
export(sortInputTaxa)
export(sortTaxaFromTree)
export(taxonomyTableCreator)
export(varDistTaxPlot)
export(wideToLong)
export(xmlParser)
import(BiocStyle)
import(bsplus)
import(data.table, except = c(first, last, between))
import(dplyr)
import(ggplot2)
import(plotly, except = c(last_plot, select))
import(scattermore)
import(svglite)
import(tsne)
import(umap)
importFrom(Biostrings,readAAStringSet)
importFrom(DT,dataTableOutput)
importFrom(DT,renderDataTable)
importFrom(RColorBrewer,brewer.pal)
importFrom(RColorBrewer,brewer.pal.info)
importFrom(RCurl,url.exists)
importFrom(Rfast,Dist)
importFrom(ape,as.phylo)
importFrom(ape,plot.phylo)
importFrom(ape,read.tree)
importFrom(ape,root)
importFrom(bioDist,cor.dist)
importFrom(bioDist,mutualInfo)
importFrom(colourpicker,colourInput)
importFrom(data.table,dcast)
importFrom(data.table,fread)
importFrom(data.table,melt)
importFrom(data.table,rbindlist)
importFrom(data.table,setDT)
importFrom(data.table,setnames)
importFrom(data.table,transpose)
importFrom(dplyr,add_count)
importFrom(dplyr,arrange)
importFrom(dplyr,count)
importFrom(dplyr,distinct)
importFrom(dplyr,filter)
importFrom(dplyr,group_by)
importFrom(dplyr,inner_join)
importFrom(dplyr,left_join)
importFrom(dplyr,mutate)
importFrom(dplyr,select)
importFrom(dplyr,summarise)
importFrom(dplyr,tibble)
importFrom(energy,dcor)
importFrom(fastcluster,hclust)
importFrom(grDevices,colorRampPalette)
importFrom(grid,convertHeight)
importFrom(grid,grobHeight)
importFrom(grid,unit)
importFrom(gridExtra,arrangeGrob)
importFrom(gridExtra,grid.arrange)
importFrom(htmlwidgets,saveWidget)
importFrom(pbapply,pblapply)
importFrom(shinyFiles,parseDirPath)
importFrom(shinyFiles,shinyDirButton)
importFrom(shinyFiles,shinyDirChoose)
importFrom(shinycssloaders,withSpinner)
importFrom(shinyjs,disable)
importFrom(shinyjs,enable)
importFrom(shinyjs,html)
importFrom(shinyjs,reset)
importFrom(shinyjs,toggleState)
importFrom(stats,aggregate)
importFrom(stats,as.dist)
importFrom(stats,complete.cases)
importFrom(stats,dist)
importFrom(stats,na.omit)
importFrom(stringr,str_count)
importFrom(stringr,str_split_fixed)
importFrom(stringr,str_wrap)
importFrom(utils,head)
importFrom(utils,tail)
importFrom(xml2,read_xml)
importFrom(xml2,xml_attr)
importFrom(xml2,xml_find_all)
importFrom(yaml,read_yaml)
importFrom(yaml,write_yaml)
importFrom(zoo,na.locf)
