% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/parsePhyloProfile.R
\name{getInputTaxaID}
\alias{getInputTaxaID}
\title{Get ID list of input taxa from the main input}
\usage{
getInputTaxaID(rawProfile = NULL)
}
\arguments{
\item{rawProfile}{A dataframe of input phylogenetic profile in long format}
}
\value{
List of all input taxon IDs (e.g. ncbi1234). Default = NULL.
}
\description{
Get ID list of input taxa from the main input
}
\examples{
data("mainLongRaw", package="PhyloProfile")
getInputTaxaID(mainLongRaw)
}
\seealso{
\code{\link{createLongMatrix}}, \code{\link{mainLongRaw}}
}
\author{
Vinh Tran tran@bio.uni-frankfurt.de
}
